/*
 * $Id: HtmlTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 2007 Engineering Ingegneria Informatica S.p.A.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * Renders an HTML <html> element with appropriate language attributes if
 * there is a current Locale available in the user's session.
 *
 * @version $Rev: 376841 $ $Date: 2005-08-21 19:08:45 -0400 (Sun, 21 Aug 2005)
 *          $
 */
public class HtmlTag extends TagSupport {
    // ------------------------------------------------------------- Properties

	private static final long serialVersionUID = 1L;

    /**
     * Are we rendering an xhtml page?
     */
    protected boolean xhtml = false;

    /**
     * Are we rendering a lang attribute?
     *
     * @since Struts 1.2
     */
    protected boolean lang = false;

    public boolean getXhtml() {
        return this.xhtml;
    }

    public void setXhtml(boolean xhtml) {
        this.xhtml = xhtml;
    }

    /**
     * Returns true if the tag should render a lang attribute.
     *
     * @since Struts 1.2
     */
    public boolean getLang() {
        return this.lang;
    }

    /**
     * Sets whether the tag should render a lang attribute.
     *
     * @since Struts 1.2
     */
    public void setLang(boolean lang) {
        this.lang = lang;
    }

    /**
     * Process the start of this tag.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        TagUtils.getInstance().write(this.pageContext,
            this.renderHtmlStartElement());

        return EVAL_BODY_INCLUDE;
    }

    /**
     * Renders an &lt;html&gt; element with appropriate language attributes.
     *
     * @since Struts 1.2
     */
    protected String renderHtmlStartElement() {
        StringBuffer sb = new StringBuffer("<html");

        String language = null;
        String country = "";

        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);

        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "HtmlTag::renderHtmlStartElement:: requestContainer nullo");
        } else {
            SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
            language = (String)permanentContainer.getAttribute(it.eng.spago.base.Constants.USER_LANGUAGE);
            country = (String)permanentContainer.getAttribute(it.eng.spago.base.Constants.USER_COUNTRY);
        }

        boolean validLanguage = ((language != null) && (language.length() > 0));
        boolean validCountry = ((country != null) && (country.length() > 0));

        if (this.xhtml) {
            this.pageContext.setAttribute(Constants.XHTML_KEY, "true",
                PageContext.PAGE_SCOPE);

            sb.append(" xmlns=\"http://www.w3.org/1999/xhtml\"");
        }

        if ((this.lang || this.xhtml) && validLanguage) {
            sb.append(" lang=\"");
            sb.append(language);

            if (validCountry) {
                sb.append("-");
                sb.append(country);
            }

            sb.append("\"");
        }

        if (this.xhtml && validLanguage) {
            sb.append(" xml:lang=\"");
            sb.append(language);

            if (validCountry) {
                sb.append("-");
                sb.append(country);
            }

            sb.append("\"");
        }

        sb.append(">");

        return sb.toString();
    }

    /**
     * Process the end of this tag.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doEndTag() throws JspException {
        TagUtils.getInstance().write(pageContext, "</html>");

        // Evaluate the remainder of this page
        return (EVAL_PAGE);
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        this.xhtml = false;
        this.lang = false;
    }
}
