/*
 * $Id: ImageTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 1999-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.alias.AliasService;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.commons.validator.GenericValidator;


/**
 * Tag for input fields of type "image".
 *
 * @version $Rev: 376841 $ $Date: 2005-04-26 20:11:47 -0400 (Tue, 26 Apr 2005)
 *          $
 */
public class ImageTag extends SubmitTag {
    // ------------------------------------------------------------- Properties

	private static final long serialVersionUID = 1L;
	
	protected String alias = null;
	protected String aliasRelated = null;
	
    /**
     * The alignment for this image.
     */
    protected String align = null;

    /**
     * The border size around the image.
     */
    protected String border = null;

    /**
     * The URL of this image.
     */
    protected String src = null;

    /**
     * The message resources key for the URL of this image.
     */
    protected String srcKey = null;

    // --------------------------------------------------------- Constructor
    public ImageTag() {
        super();
        property = "";
    }

	public void setAlias(String alias) {
		this.alias = alias;
	}

	public void setAliasRelated(String aliasRelated) {
		this.aliasRelated = aliasRelated;
	}

    /**
     * @deprecated Align attribute is deprecated in HTML 4.x.
     */
    public String getAlign() {
        return (this.align);
    }

    /**
     * @deprecated Align attribute is deprecated in HTML 4.x.
     */
    public void setAlign(String align) {
        this.align = align;
    }

    public String getBorder() {
        return (this.border);
    }

    public void setBorder(String border) {
        this.border = border;
    }

    public String getSrc() {
        return (this.src);
    }

    public void setSrc(String src) {
        this.src = src;
    }

    public String getSrcKey() {
        return (this.srcKey);
    }

    public void setSrcKey(String srcKey) {
        this.srcKey = srcKey;
    }

    // --------------------------------------------------------- Protected Methods

    /**
     * Render the opening element.
     *
     * @return The opening part of the element.
     */
    protected String getElementOpen() {
        return "<input type=\"image\"";
    }

    /**
     * Render the button attributes
     *
     * @param results The StringBuffer that output will be appended to.
     */
    protected void prepareButtonAttributes(StringBuffer results)
        throws JspException {
        String tmp = src();

        if (tmp != null) {
            HttpServletResponse response =
                (HttpServletResponse) pageContext.getResponse();

            prepareAttribute(results, "src", response.encodeURL(tmp));
        }

        prepareAttribute(results, "align", getAlign());
        prepareAttribute(results, "border", getBorder());
        prepareAttribute(results, "value", getValue());
        prepareAttribute(results, "accesskey", getAccesskey());
        prepareAttribute(results, "tabindex", getTabindex());
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        property = "";
        src = null;
        srcKey = null;
    }

    // ------------------------------------------------------ Protected Methods

    /**
     * Return the base source URL that will be rendered in the
     * <code>src</code> property for this generated element, or
     * <code>null</code> if there is no such URL.
     *
     * @throws JspException if an error occurs
     */
    protected String src() throws JspException {

        // Deal with an absolute source that has been specified
        String configAlias = null;
        
        if (!GenericValidator.isBlankOrNull(alias)) {
        	try {
	        	AliasService aliasService = (AliasService)ServiceCatalog.getInstance()
	        		.getService(TagUtils.getInstance()
	        				.getRequestContext(pageContext), "ALIAS");
	        	
		        configAlias = aliasService.getAlias(alias);
        	} catch (Exception ex) {
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
        			"ImageTag::src: Error retrieving 'alias service' or 'alias'", ex);
        		throw new JspException("Error retrieving alias", ex);
        	}
        }
        if (!GenericValidator.isBlankOrNull(configAlias)) {
        	StringBuffer url = new StringBuffer();
        	url.append(configAlias);
        	if (!GenericValidator.isBlankOrNull(aliasRelated)) {
        		url.append(aliasRelated);
        	}
        	return url.toString();
        } else if (this.src != null) {
            if (this.srcKey != null) {
	        	TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO, TracerSingleton.WARNING,
		        		"ImageTag::src: both src and srcKey are not null");
                JspException e =
                    new JspException(messages.getMessage("imgTag.src"));

                TagUtils.getInstance().saveException(pageContext, e);
                throw e;
            }

            return (this.src);
        }

        // Deal with an indirect source that has been specified
        if (this.srcKey == null) {
        	TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO, TracerSingleton.WARNING,
	        		"ImageTag::src: both src and srcKey are null");
            JspException e =
                new JspException(messages.getMessage("imgTag.src"));

            TagUtils.getInstance().saveException(pageContext, e);
            throw e;
        }

        return TagUtils.getInstance().message(getBundle(), this.srcKey);
    }
}
