/*
 * $Id: ImgTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 1999-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.SourceBean;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.alias.AliasService;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.commons.validator.GenericValidator;


/**
 * <p>Generate an IMG tag to the specified image URI. </p>
 *
 * <p>TODO:</p>
 *
 * <ul>
 *
 * <li>Make the <strong>alt</strong> and <strong>src</strong> settable from
 * properties (for i18n)</li>
 *
 * </ul>
 *
 * @version $Rev: 376841 $
 */
public class ImgTag extends BaseHandlerTag {
    /**
     * The message resources for this package.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(Constants.Package
            + ".LocalStrings");

    // ------------------------------------------------------------- Properties

    private static final long serialVersionUID = 1L;
    
    protected String alias = null;
    protected String aliasRelated = null;
    
    /**
     * The property to specify where to align the image.
     */
    protected String align = null;

    /**
     * The border size around the image.
     */
    protected String border = null;

    /**
     * The image height.
     */
    protected String height = null;

    /**
     * The horizontal spacing around the image.
     */
    protected String hspace = null;

    /**
     * The image name for named images.
     */
    protected String imageName = null;

    /**
     * Server-side image map declaration.
     */
    protected String ismap = null;

    /**
     * In situations where an image is dynamically generated (such as to
     * create a chart graph), this specifies the single-parameter request
     * parameter name to generate.
     */
    protected String paramId = null;

    /**
     * The single-parameter JSP bean name.
     */
    protected String paramName = null;

    /**
     * The single-parameter JSP bean scope.
     */
    protected String paramScope = null;

    /**
     * The JSP bean property name for query parameters.
     */
    protected String property = null;

    /**
     * The scope of the bean specified by the name property, if any.
     */
    protected String scope = null;

    /**
     * The image source URI.
     */
    protected String src = null;

    /**
     * The message resources key under which we should look up the
     * <code>src</code> attribute for this generated tag, if any.
     */
    protected String srcKey = null;

    /**
     * Client-side image map declaration.
     */
    protected String usemap = null;

    /**
     * The vertical spacing around the image.
     */
    protected String vspace = null;

    /**
     * The image width.
     */
    protected String width = null;
    protected boolean useLocalEncoding = false;

    // ----------------------------------------------------- Constructor
    public ImgTag() {
        super();
        doDisabled = false;
    }

	public void setAlias(String alias) {
		this.alias = alias;
	}

	public void setAliasRelated(String aliasRelated) {
		this.aliasRelated = aliasRelated;
	}
	
    public String getAlign() {
        return (this.align);
    }

    public void setAlign(String align) {
        this.align = align;
    }

    public String getBorder() {
        return (this.border);
    }

    public void setBorder(String border) {
        this.border = border;
    }

    public String getHeight() {
        return (this.height);
    }

    public void setHeight(String height) {
        this.height = height;
    }

    public String getHspace() {
        return (this.hspace);
    }

    public void setHspace(String hspace) {
        this.hspace = hspace;
    }

    public String getImageName() {
        return (this.imageName);
    }

    public void setImageName(String imageName) {
        this.imageName = imageName;
    }

    public String getIsmap() {
        return (this.ismap);
    }

    public void setIsmap(String ismap) {
        this.ismap = ismap;
    }

    public String getParamId() {
        return (this.paramId);
    }

    public void setParamId(String paramId) {
        this.paramId = paramId;
    }

    public String getParamName() {
        return (this.paramName);
    }

    public void setParamName(String paramName) {
        this.paramName = paramName;
    }

    public String getParamScope() {
        return (this.paramScope);
    }

    public void setParamScope(String paramScope) {
        this.paramScope = paramScope;
    }

    public String getProperty() {
        return (this.property);
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public String getScope() {
        return (this.scope);
    }

    public void setScope(String scope) {
        this.scope = scope;
    }

    public String getSrc() {
        return (this.src);
    }

    public void setSrc(String src) {
        this.src = src;
    }

    public String getSrcKey() {
        return (this.srcKey);
    }

    public void setSrcKey(String srcKey) {
        this.srcKey = srcKey;
    }

    public String getUsemap() {
        return (this.usemap);
    }

    public void setUsemap(String usemap) {
        this.usemap = usemap;
    }

    public String getVspace() {
        return (this.vspace);
    }

    public void setVspace(String vspace) {
        this.vspace = vspace;
    }

    public String getWidth() {
        return (this.width);
    }

    public void setWidth(String width) {
        this.width = width;
    }

    public boolean isUseLocalEncoding() {
        return useLocalEncoding;
    }

    public void setUseLocalEncoding(boolean b) {
        useLocalEncoding = b;
    }

    // --------------------------------------------------------- Public Methods

    /**
     * Render the beginning of the IMG tag.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        // Evaluate the body of this tag
        return (EVAL_BODY_AGAIN);
    }

    /**
     * Render the end of the IMG tag.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doEndTag() throws JspException {
        // Generate the name definition or image element
        HttpServletResponse response =
            (HttpServletResponse) pageContext.getResponse();
        StringBuffer results = new StringBuffer("<img");
        String tmp = src();
        String srcurl = url(tmp);

        if (srcurl != null) {
            prepareAttribute(results, "src", response.encodeURL(srcurl));
        }

        prepareAttribute(results, "name", getImageName());
        prepareAttribute(results, "height", getHeight());
        prepareAttribute(results, "width", getWidth());
        prepareAttribute(results, "align", getAlign());
        prepareAttribute(results, "border", getBorder());
        prepareAttribute(results, "hspace", getHspace());
        prepareAttribute(results, "vspace", getVspace());
        prepareAttribute(results, "ismap", getIsmap());
        prepareAttribute(results, "usemap", getUsemap());
        results.append(prepareStyles());
        results.append(prepareEventHandlers());
        prepareOtherAttributes(results);
        results.append(getElementClose());

        TagUtils.getInstance().write(pageContext, results.toString());

        return (EVAL_PAGE);
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();

        border = null;
        height = null;
        hspace = null;
        imageName = null;
        ismap = null;
        paramId = null;
        paramName = null;
        paramScope = null;
        property = null;
        scope = null;
        src = null;
        srcKey = null;
        usemap = null;
        vspace = null;
        width = null;
    }

    // ------------------------------------------------------ Protected Methods

    /**
     * Convenience method to throw a "imgTag.src" exception.
     *
     * @throws JspException
     */
    private void throwImgTagSrcException()
        throws JspException {
        JspException e = new JspException(messages.getMessage("imgTag.src"));

        TagUtils.getInstance().saveException(pageContext, e);
        throw e;
    }

    /**
     * Return the base source URL that will be rendered in the
     * <code>src</code> property for this generated element, or
     * <code>null</code> if there is no such URL.
     *
     * @throws JspException if an error occurs
     */
    protected String src() throws JspException {

        // Deal with an absolute source that has been specified
        String configAlias = null;
        
        if (!GenericValidator.isBlankOrNull(alias)) {
        	try {
	        	AliasService aliasService = (AliasService)ServiceCatalog.getInstance()
	        		.getService(TagUtils.getInstance()
	        				.getRequestContext(pageContext), "ALIAS");
	        	
		        configAlias = aliasService.getAlias(alias);
        	} catch (Exception ex) {
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
        			"ImgTag::src: Error retrieving 'alias service' or 'alias'", ex);
        		throw new JspException("Error retrieving alias", ex);
        	}
        }
        if (!GenericValidator.isBlankOrNull(configAlias)) {
        	StringBuffer url = new StringBuffer();
        	url.append(configAlias);
        	if (!GenericValidator.isBlankOrNull(aliasRelated)) {
        		url.append(aliasRelated);
        	}
        	return url.toString();
        } else if (this.src != null) {
            if (this.srcKey != null) {
	        	TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO, TracerSingleton.WARNING,
	        		"ImgTag::src: both src and srcKey are not null");
                throwImgTagSrcException();
            }

            return (this.src);
        }

        // Deal with an indirect source that has been specified
        if (this.srcKey == null) {
        	TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO, TracerSingleton.WARNING,
	        		"ImgTag::src: both src and srcKey are null");
            throwImgTagSrcException();
        }

        return TagUtils.getInstance().message(getBundle(), this.srcKey);
    }

    /**
     * Return the specified src URL, modified as necessary with optional
     * request parameters.
     *
     * @param url The URL to be modified (or null if this url will not be
     *            used)
     * @throws JspException if an error occurs preparing the URL
     */
    protected String url(String url)
        throws JspException {
        if (url == null) {
            return (url);
        }

        String charEncoding = "UTF-8";

        if (useLocalEncoding) {
            charEncoding = pageContext.getResponse().getCharacterEncoding();
        }

        // Start with an unadorned URL as specified
        StringBuffer src = new StringBuffer(url);

        // Append a single-parameter name and value, if requested
        if ((paramId != null)) {
            if (src.toString().indexOf('?') < 0) {
                src.append('?');
            } else {
                src.append("&amp;");
            }

            src.append(paramId);
            src.append('=');

            Object value =
                TagUtils.getInstance().lookupProperty(pageContext, paramName, paramScope);

            if (value != null) {
                src.append(TagUtils.getInstance().encodeURL(value.toString(),
                        charEncoding));
            }
        }

        // Just return the URL if there is no property to look for
        if (property == null) {
            return (src.toString());
        }

//      // Append the required query parameters
        boolean question = (src.toString().indexOf("?") >= 0);
        Iterator queryPairs = TagUtils.getInstance()
        	.lookupMultipleProperty(pageContext, property, scope).iterator();
        
        while (queryPairs.hasNext()) {
        	SourceBean queryPair = (SourceBean)queryPairs.next();
        	String key = (String)queryPair.getAttribute("NAME");
        	String value = (String)queryPair.getAttribute("VALUE");

            if (value == null) {
                if (question) {
                    src.append("&amp;");
                } else {
                    src.append('?');
                    question = true;
                }

                src.append(key);
                src.append('=');

            } else {
                if (question) {
                    src.append("&amp;");
                } else {
                    src.append('?');
                    question = true;
                }

                src.append(key);
                src.append('=');
                src.append(TagUtils.getInstance().encodeURL(value.toString(),
                        charEncoding));
            }
        }

        // Return the final result
        return (src.toString());
    }

}
