/*
 * $Id: LinkTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 2007 Engineering Ingegneria Informatica S.p.A.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.jsp.JspException;


/**
 * Generate a URL-encoded hyperlink to the specified URI.
 *
 * @version $Rev: 376841 $ $Date: 2005-04-06 02:37:00 -0400 (Wed, 06 Apr 2005)
 *          $
 */
public class LinkTag extends BaseHandlerTag {
    /**
     * The message resources for this package.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(Constants.Package
            + ".LocalStrings");

    // ----------------------------------------------------- Instance Variables

    protected String alias = null;
    protected String aliasRelated = null;

    /**
     * The body content of this tag (if any).
     */
    protected String text = null;

    // ------------------------------------------------------------- Properties

    private static final long serialVersionUID = 1L;
    
    /**
     * The anchor to be added to the end of the generated hyperlink.
     */
    protected String anchor = null;

    /**
     * The hyperlink URI.
     */
    protected String href = null;

    /**
     * The link name for named links.
     */
    protected String linkName = null;

    /**
     * The single-parameter request parameter name to generate.
     */
    protected String paramId = null;

    /**
     * The single-parameter JSP bean name.
     */
    protected String paramName = null;

    /**
     * The single-parameter JSP bean scope.
     */
    protected String paramScope = null;

    /**
     * The JSP bean property name for query parameters.
     */
    protected String property = null;

    /**
     * The scope of the bean specified by the name property, if any.
     */
    protected String scope = null;

    /**
     * The window target.
     */
    protected String target = null;

    /**
     * Include transaction token (if any) in the hyperlink?
     */
    protected boolean transaction = false;

    /**
     * Name of parameter to generate to hold index number
     */
    protected String indexId = null;
    protected boolean useLocalEncoding = false;

    // ----------------------------------------------------- Constructor
    public LinkTag() {
        super();
        doDisabled = false;
    }

    public String getAnchor() {
        return (this.anchor);
    }

    public void setAnchor(String anchor) {
        this.anchor = anchor;
    }

    public String getHref() {
        return (this.href);
    }

    public void setHref(String href) {
        this.href = href;
    }

    public String getLinkName() {
        return (this.linkName);
    }

    public void setLinkName(String linkName) {
        this.linkName = linkName;
    }

    public String getParamId() {
        return (this.paramId);
    }

    public void setParamId(String paramId) {
        this.paramId = paramId;
    }

    public String getParamName() {
        return (this.paramName);
    }

    public void setParamName(String paramName) {
        this.paramName = paramName;
    }

    public String getParamScope() {
        return (this.paramScope);
    }

    public void setParamScope(String paramScope) {
        this.paramScope = paramScope;
    }

    public String getProperty() {
        return (this.property);
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public String getScope() {
        return (this.scope);
    }

    public void setScope(String scope) {
        this.scope = scope;
    }

    public String getTarget() {
        return (this.target);
    }

    public void setTarget(String target) {
        this.target = target;
    }

    public boolean getTransaction() {
        return (this.transaction);
    }

    public void setTransaction(boolean transaction) {
        this.transaction = transaction;
    }

    public String getIndexId() {
        return (this.indexId);
    }

    public void setIndexId(String indexId) {
        this.indexId = indexId;
    }

    public boolean isUseLocalEncoding() {
        return useLocalEncoding;
    }

    public void setUseLocalEncoding(boolean b) {
        useLocalEncoding = b;
    }

	public void setAlias(String alias) {
		this.alias = alias;
	}

	public void setAliasRelated(String aliasRelated) {
		this.aliasRelated = aliasRelated;
	}
    // --------------------------------------------------------- Public Methods

    /**
     * Render the beginning of the hyperlink. <p> Support for indexed property
     * since Struts 1.1
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        // Generate the opening anchor element
        StringBuffer results = new StringBuffer("<a");

        // Special case for name anchors
        prepareAttribute(results, "name", getLinkName());

        if ((getLinkName() == null) || (getHref() != null)) {
            prepareAttribute(results, "href", calculateURL());
        }

        prepareAttribute(results, "target", getTarget());
        prepareAttribute(results, "accesskey", getAccesskey());
        prepareAttribute(results, "tabindex", getTabindex());
        results.append(prepareStyles());
        results.append(prepareEventHandlers());
        prepareOtherAttributes(results);
        results.append(">");

        TagUtils.getInstance().write(pageContext, results.toString());

        // Evaluate the body of this tag
        this.text = null;

        return (EVAL_BODY_AGAIN);
    }

    /**
     * Save the associated label from the body content.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doAfterBody() throws JspException {
        if (bodyContent != null) {
            String value = bodyContent.getString().trim();

            if (value.length() > 0) {
                text = value;
            }
        }

        return (SKIP_BODY);
    }

    /**
     * Render the end of the hyperlink.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doEndTag() throws JspException {
        // Prepare the textual content and ending element of this hyperlink
        StringBuffer results = new StringBuffer();

        if (text != null) {
            results.append(text);
        }

        results.append("</a>");

        TagUtils.getInstance().write(pageContext, results.toString());

        return (EVAL_PAGE);
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        anchor = null;
        href = null;
        linkName = null;
        paramId = null;
        paramName = null;
        paramScope = null;
        property = null;
        scope = null;
        target = null;
        text = null;
        transaction = false;
        indexId = null;
        useLocalEncoding = false;
    }

    // ------------------------------------------------------ Protected Methods

    /**
     * Return the complete URL to which this hyperlink will direct the user.
     * Support for indexed property since Struts 1.1
     *
     * @throws JspException if an exception is thrown calculating the value
     */
    protected String calculateURL()
        throws JspException {
        // Identify the parameters we will add to the completed URL
        List params =
            TagUtils.getInstance().computeParameters(pageContext, paramId,
                paramName, paramScope, property, scope, transaction);

        // if "indexed=true", add "index=x" parameter to query string
        // * @since Struts 1.1
        if (indexed) {
            int indexValue = getIndexValue();

            //calculate index, and add as a parameter
            if (params == null) {
                params = new ArrayList(); //create new HashMap if no other params
            }

            try {
	            SourceBean indexParam = new SourceBean("INDEX");
	            if (indexId != null) {
	            	indexParam.setAttribute("NAME", indexId);
	            } else {
	            	indexParam.setAttribute("NAME", "index");
	            }
	            indexParam.setAttribute("VALUE", Integer.toString(indexValue));
	            params.add(indexParam);
            } catch (SourceBeanException sbe) {
            	TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO,
            			TracerSingleton.WARNING, "Error creating SourceBean", sbe);
            }
        }

        String url = null;

        try {
            url = TagUtils.getInstance().computeURLWithCharEncoding(pageContext,
                    href, params, anchor, false, alias, aliasRelated,
                    useLocalEncoding, false);
        } catch (MalformedURLException e) {
            TagUtils.getInstance().saveException(pageContext, e);
            throw new JspException(messages.getMessage("rewrite.url",
                    e.toString()));
        }

        return (url);
    }
}
