/*
 * $Id: MessageTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 1999-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.message.MessageBundle;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.taglib.TagUtils;

import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

/**
 * Custom tag that iterates the elements of a message collection. It defaults
 * to retrieving the messages from <code>Globals.ERROR_KEY</code>, but if the
 * message attribute is set to true then the messages will be retrieved from
 * <code>Globals.MESSAGE_KEY</code>. This is an alternative to the default
 * <code>ErrorsTag</code>.
 *
 * @version $Rev: 376841 $ $Date: 2005-11-08 23:50:53 -0500 (Tue, 08 Nov 2005)
 *          $
 * @since Struts 1.1
 * 
 * @deprecated use it.eng.spago.tags.MessageTag
 */
public class MessageTag extends BodyTagSupport {
    /**
     * The message resources for this package.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(Constants.Package
            + ".LocalStrings");

    /**
     * Iterator of the elements of this error collection, while we are
     * actually running.
     */
    protected Iterator iterator = null;

    /**
     * Whether or not any error messages have been processed.
     */
    protected boolean processed = false;

    /**
     * The name of the scripting variable to be exposed.
     */
    protected String id = null;

    /**
     * The servlet context attribute key for our resources.
     */
    protected String bundle = null;

    /**
     * The request attribute key for our error messages (if any).
     */
    protected String name = Constants.ERROR_KEY;

    /**
     * Says if messages retrieved are resources to search on bundle.
     */
    protected String areResources = "false";
    
    /**
     * The name of the property for which error messages should be returned,
     * or <code>null</code> to return all errors.
     */
    protected String property = null;

    /**
     * The message resource key for errors header.
     */
    protected String header = null;

    /**
     * The message resource key for errors footer.
     */
    protected String footer = null;

    /**
     * If this is set to 'true', then the <code>Globals.MESSAGE_KEY</code>
     * will be used to retrieve the messages from scope.
     */
    protected String message = null;

    public String getId() {
        return (this.id);
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getBundle() {
        return (this.bundle);
    }

    public void setBundle(String bundle) {
        this.bundle = bundle;
    }

    public String getName() {
        return (this.name);
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getProperty() {
        return (this.property);
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public String getHeader() {
        return (this.header);
    }

    public void setHeader(String header) {
        this.header = header;
    }

    public String getFooter() {
        return (this.footer);
    }

    public void setFooter(String footer) {
        this.footer = footer;
    }

    public String getMessage() {
        return (this.message);
    }

    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * Construct an iterator for the specified collection, and begin looping
     * through the body once per element.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
//    	// TO DO
//        // Initialize for a new request.
//        processed = false;
//
//        List messages = null;
//        
//        try {
//            messages =
//                TagUtils.getInstance().lookupMultipleProperty
//                	(pageContext, name, null);
//        } catch (JspException e) {
//            TagUtils.getInstance().saveException(pageContext, e);
//            throw e;
//        }
//
//        // Store the first value and evaluate, or skip the body if none
//        if (messages == null) {
//            return SKIP_BODY;
//        }
//
//        this.iterator = messages.iterator();
//
//        // process the first message
//        processMessage((ActionMessage) iterator.next());
//
//        if ((header != null) && (header.length() > 0)) {
//            String headerMessage =
//                TagUtils.getInstance().message(bundle, header);
//
//            if (headerMessage != null) {
//                TagUtils.getInstance().write(headerMessage);
//            }
//        }
//
//        // Set the processed variable to true so the
//        // doEndTag() knows processing took place
//        processed = true;
//
        return (EVAL_BODY_TAG);
    }

    /**
     * Make the next collection element available and loop, or finish the
     * iterations if there are no more elements.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doAfterBody() throws JspException {
        // Render the output from this iteration to the output stream
        if (bodyContent != null) {
            TagUtils.getInstance().writePrevious(pageContext,
                bodyContent.getString());
            bodyContent.clearBody();
        }

        // Decide whether to iterate or quit
        if (iterator.hasNext()) {
        	// TO DO
//            processMessage((ActionMessage) iterator.next());

            return (EVAL_BODY_TAG);
        } else {
            return (SKIP_BODY);
        }
    }

    /**
     * Process a message.
     */
    private void processMessage(String report)
        throws JspException {
        String msg = null;

//        if (Boolean.parseBoolean(areResources)) {
//            msg = TagUtils.getInstance().message(pageContext, bundle,
//                    report.getKey(), report.getValues());
//
//            if (msg == null) {
//                String bundleName = (bundle == null) ? "default" : bundle;
//
//                msg = messageResources.getMessage("messagesTag.notfound",
//                        report.getKey(), bundleName);
//            }
//        } else {
//            msg = report.getKey();
//        }
//
//        if (msg == null) {
//            pageContext.removeAttribute(id);
//        } else {
//            pageContext.setAttribute(id, msg);
//        }
    }

    /**
     * Clean up after processing this enumeration.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doEndTag() throws JspException {
        if (processed && (footer != null) && (footer.length() > 0)) {
            String footerMessage =
            	// TO DO
//                TagUtils.getInstance().message(pageContext, bundle,
//                        footer);
            MessageBundle.getMessage(bundle + "." + footer);

            if (footerMessage != null) {
                TagUtils.getInstance().write(pageContext, footerMessage);
            }
        }

        return EVAL_PAGE;
    }

    /**
     * Release all allocated resources.
     */
    public void release() {
        super.release();
        iterator = null;
        processed = false;
        id = null;
        bundle = null;
        name = Constants.ERROR_KEY;
        property = null;
        header = null;
        footer = null;
        message = null;
    }
}
