/*
 *
 * Copyright 2007 Engineering Ingegneria Informatica S.p.A.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.Constants;
import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.alias.AliasService;
import it.eng.spago.services.http.HttpService;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import java.net.URL;

import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.validator.GenericValidator;

public class Snipplet extends TagSupport {
	private static final long serialVersionUID = 1L;

    private String _url;
    private String _alias;

    /**
     *  Description of the Method
     *
     *@return                                     Description of the Return
     *      Value
     *@exception  javax.servlet.jsp.JspException  Description of the Exception
     */
    public int doStartTag() throws javax.servlet.jsp.JspException {
        try {
            HttpService service = (HttpService) ServiceCatalog.getInstance()
            	.getService(TagUtils.getInstance().getRequestContext(pageContext),
				            this.getId());

            if (!GenericValidator.isBlankOrNull(_alias)) {
            	String value = null;
            	try {
            		AliasService aliasService = (AliasService)ServiceCatalog.getInstance()
            			.getService(TagUtils.getInstance().getRequestContext(pageContext),
            					"ALIAS");

		        	value = aliasService.getAlias(_alias);
		            if (value.startsWith("http:/")){
		                pageContext.getOut().print(service.doGet(new URL(value)));
		            }
            	} catch (Exception ex) {
            		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                		"Snipplet::doStartTag: Error retrieving 'alias service' or 'alias'", ex);
            	}

            } else {
              pageContext.getOut().print(service.doGet(_url));
            }

        } catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Snipplet::doStartTag", ex);
        }
        return EVAL_PAGE;
    }

    /**
     *  Sets the url attribute of the Snipplet object
     *
     *@param  url  The new url value
     */
    public void setUrl(String url) {
        this._url = url;
    }

    /**
     *  Gets the url attribute of the Snipplet object
     *
     *@return    The url value
     */
    public String getUrl() {
        return _url;
    }

    /**
     *  Sets the alias attribute of the Snipplet object
     *
     * @param  alias. The new alias value
     */
    public void setAlias(String alias) {
        this._alias = alias;
    }

    /**
     *  Gets the alias attribute of the Snipplet object
     *
     *@return    The alias value
     */
    public String getAlias() {
        return _alias;
    }

}
