/**

Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.taglib.sourcebean;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.validator.GenericValidator;

import it.eng.spago.base.SourceBean;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Write the <code>String</code> value of an SourceBean present in PageContext.
 * 
 * @author buso
 *
 */
public class GetValueTag extends TagSupport {
	public static final String PAGE_CONTEXT_SCOPE = "PAGE_CONTEXT".intern();

    /**
     * The message resources for this package.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(
            Constants.PACKAGE_NAME + ".LocalStrings");
	
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /**
     * The id used to retrive the data from PageContext.
     */
    private String _id = null;
    
    /**
     * If valorized the object retrieved with <code>id</code> is treated
	 * as a <code>SourceBean</code> and the tag will return the
	 * <code>SourceBean</code> attribute identified by property.
     */
    private String _property = null;
    
    /**
     * Scope where to look for data identified by <code>property</code>. Possible
     * values are:
	 * <ul>
	 * 	<li>PAGE_CONTEXT</li>
	 * 	<li>CONFIG</li>
	 * 	<li>USER</li>
	 * 	<li>ADAPTER_REQUEST</li>
	 * 	<li>SERVICE_REQUEST</li>
	 * 	<li>SESSION</li>
	 * 	<li>APPLICATION</li>
	 * 	<li>ADAPTER_RESPONSE</li>
	 * 	<li>SERVICE_RESPONSE</li>
	 * 	<li>ERROR</li>
	 * </ul>
     */
    private String _scope = PAGE_CONTEXT_SCOPE;

	/**
	 * Constructor for GetValueTag.
	 */
	public GetValueTag() {
		super();
	}

    public int doStartTag() throws JspException {
    	String value = null;
    	
    	if (_id == null) {
    		// TODO add this message to resource bundle
            throw new JspException(messages.getMessage("write.id"));
    	}
    	
    	Object objValue = null;
    	if (_scope.equals(PAGE_CONTEXT_SCOPE)) {
    		objValue = pageContext.getAttribute(_id);
    	} else {
    		objValue = TagUtils.getInstance().lookupProperty(pageContext,
    			_id, _scope);
    	}
    	if (objValue != null) {
    		if (!GenericValidator.isBlankOrNull(_property)
    			&& _scope.equals(PAGE_CONTEXT_SCOPE)) {
    			objValue = ((SourceBean)objValue).getAttribute(_property);
    		}
    		if (objValue instanceof String) {
    			value = (String)objValue;
    		} else if (objValue != null) {
    			value = objValue.toString();
    		}
    	}
        
        if (value == null)
       			value = "";
        	
        StringBuffer _htmlStream = new StringBuffer();        
        _htmlStream.append(value);
        
        try {
            pageContext.getOut().print(_htmlStream);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(it.eng.spago.base.Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ReloadFieldTag::doStartTag:: Impossibile inviare lo stream");
            throw new JspException("Impossibile inviare lo stream");
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    public void setId(String id) {
    	_id = id;
    }
    
    public void setProperty(String prop) {
    	_property = prop;
    }

	public String getScope() {
		return _scope;
	}

	public void setScope(String _scope) {
		this._scope = _scope;
	}
}
